const IMILib2017Common = require('../imi-lib-2017-common');

function VocabReform (ivd, externalVocabulary = null, bc = false) {
    var vocabulary = this.vocabulary = {
        "targetNamespace": "",
        "metadata": {},
        "prefix": [],
        "classes": [],
        "properties": [],
        "info": []
    };

    //以下で語彙を調整する
    var obj = new IMILib2017Common.parse_v(ivd);

    //TEST
    //console.log(obj);
    //TEST

    //クラス順序保存用
    var classQue = [];
    //クラスのセット文の生成用一時変数
    var classTemp = {};
    //セット文処理中にクラスに登録できなかった漏れたセット文のArray
    var leakSets = [];
    //プロパティの中の名前空間のリスト
    var prefixList = [];

    //語彙定義JSONを順に読んで処理
    for(var i = 0; i < obj.length; i++) {
        switch (obj[i].type) {
            //クラスの場合
            case 'class':
                //クラス一時保存変数にクラスを保存
                classTemp[obj[i].prefix + ':' + obj[i].name] = obj[i];
                //一時保存変数にSETを保存するArrayを作成
                classTemp[obj[i].prefix + ':' + obj[i].name]['sets'] = [];
                //クラスの順序を保存するQueに保存
                classQue.push(obj[i].prefix + ':' + obj[i].name);

                //クラスに使用されるネームスペースのチェックのための処理
                var classType = findRestriction(obj[i]['restriction'], 'type');
                if(classType.length === 0) {
                    //非推奨クラスは、後方互換プロパティがtrueの場合以外は処理しない
                    if(typeof(obj[i]['deprecated']) === 'undefined' || bc) {
                        if (obj[i]['prefix'] + ':' + obj[i]['name'] !== 'ic:概念型') {
                            //同じものは保存しない
                            if (!prefixList.includes(classType['prefix'])) {
                                //クラス中の名前空間リストに追加
                                prefixList.push(classType['prefix']);
                            }
                        }
                    }
                }
                break;
            //セット文の場合
            case 'set':
                //クラスにセット文を保存する
                try{
                    classTemp[obj[i].class.prefix + ':' + obj[i].class.name].sets.push(obj[i]);
                } catch (e) {
                    //クラスがセット文にない場合、あとで処理する配列に保存する。（IMIV語彙記法上の語彙を継承せずに拡張した場合に対処）
                    leakSets.push(obj[i]);
                    console.log(`leak set of :: ${JSON.stringify(obj[i])}`);
                }

                break;
            //プロパティの場合
            case 'property':
                //プロパティに保存する
                vocabulary.properties.push(obj[i]);

                //プロパティに使用されるネームスペースのチェックのための処理
                var propertyType = findRestriction(obj[i]['restriction'], 'type');
                var propertyPrefix = obj[i]['prefix'];
                //非推奨プロパティは、後方互換プロパティがtrueの場合以外は処理しない
                if(typeof(obj[i]['deprecated']) === 'undefined' || bc) {
                    //同じものは保存しない
                    if (!prefixList.includes(propertyType['prefix'])) {
                        //プロパティ中の名前空間リストに追加
                        prefixList.push(propertyType['prefix']);
                    }
                    if (!prefixList.includes(propertyPrefix)) {
                        //プロパティ中の名前空間リストに追加
                        prefixList.push(propertyPrefix);
                    }
                }

                break;
            //語彙の場合
            case 'vocabulary':
                //targetNamespaceを保存する
                vocabulary.targetNamespace = obj[i].data;
                //メタデータを保存する
                vocabulary.metadata = obj[i].metadata;
                break;
        }
    }

    //クラス順序保存用配列にしたがって、クラスを調整済み語彙のオブジェクトに書き込む
    for(var i = 0; i < classQue.length; i++) {
        vocabulary.classes.push(classTemp[classQue[i]]);
    }

    //使用されている名前空間のprefix一覧を読み込み、語彙のメタデータの名前空間参照情報に存在していることを確認し、クラスを調整済み語彙のオブジェクトに書き込む
    for (var i = 0; i < prefixList.length; i++) {
        //xsdはスキップ
        if(prefixList[i] === 'xsd') continue;

        var prefix = findPrefix(vocabulary.metadata, prefixList[i]);
        if (!prefix) {
            alert('語彙定義の中で使用されている名前空間が解決できませんでした。');
            throw '語彙定義の中で使用されている名前空間が解決できませんでした。'
        }
        vocabulary.prefix.push(prefix);
    }

    // 外部語彙を継承した語彙を読み込む
    if (externalVocabulary === null || typeof(externalVocabulary.baseVocabulary) === 'undefined') {
        //読み込んだ名前空間の語彙の全て
        this.baseVocabulary = {
            "prefix": [],
            "classes": [],
            "properties": []
        }
    } else {
        //読み込んだ名前空間の語彙の全て
        this.baseVocabulary = externalVocabulary.baseVocabulary;
    }

    //読み込んだ名前空間の語彙の全てへ、語彙定義テキストから読み込んだ語彙を追加する。
    ["classes", "properties", "prefix"].forEach(a => {
        this.vocabulary[a].forEach(b => {
            this.baseVocabulary[a].push(b);
        });
    });

    //セット文処理中にクラスに登録できなかった要素の登録（IMIV語彙記法上の語彙を継承せずに拡張した場合に対処）
    //TEST
    //if(leakSets.length > 0) {console.log(`this.baseVocabulary::${JSON.stringify(this.baseVocabulary)}`);}
    //TEST
    for(var i = 0; i < leakSets.length; i++) {
        var extClass = findClass(this.baseVocabulary.classes, leakSets[i]['class']['prefix'] + ':' + leakSets[i]['class']['name']);
        extClass.sets.push(leakSets[i]);
        this.vocabulary.classes.push(extClass);
    }
    return [this.vocabulary, this.baseVocabulary];
}
module.exports = VocabReform;

/**
 * メタデータをメタデータ名と言語コードで検索する
 * @param {Array<Object>} metadata 検索対象メタデータ
 * @param {String} name メタデータ名
 * @param {String} lang 言語
 * @return {string} 検索したメタデータの値
 */
function findMetadata(metadata, name, lang = 'ja') {
    //メタデータが空の場合
    if(metadata === null || typeof(metadata) === 'undefined') {
        return '';
    }
    var temp = [];
    for(var i = 0; i < metadata.length; i++) {
        if (metadata[i]['type'] === name) {
            temp.push(metadata[i]);
        }
    }
    if (temp.length === 0) {
        return '';
    }
    var targetMetadata = {};
    if (lang === 'ja') {
        for(var i = 0; i < temp.length; i++) {
            if(typeof(temp[i]['language']) === 'undefined') {
                targetMetadata = temp[i];
            }
        }
    }
    else {
        for(var i = 0; i < temp.length; i++) {
            if(temp[i]['language'] === lang) {
                targetMetadata = temp[i];
            }
        }
    }
    
    return targetMetadata['data'];
}
//module.exports = findMetadata;

/**
 * 語彙のメタデータより指定されたprefixの情報を取得する
 * @param {Array<Object>} vocabularyMetadata 語彙メタデータ
 * @param {String} prefixName 検索する名前空間名
 * @return {string} 検索したprefix
 */
function findPrefix(vocabularyMetadata, prefixName) {
    for (var i = 0; i < vocabularyMetadata.length; i++) {
        if (vocabularyMetadata[i]['type'] === 'prefix') {
            if (vocabularyMetadata[i]['prefix'] === prefixName) {
                return vocabularyMetadata[i];
            }
        }
    }
    return null;
}
//module.exports = findPrefix;

/**
 * 型制約のArrayから目的の制約を抽出
 * @param {Array<Object>} restriction 検索対象型制約Array
 * @param {String} type 制限値の種別
 * @return {Object} 検索した型制約
 */
function findRestriction(restriction, type) {
    //概念型、電話番号型
    if (typeof(restriction) === 'undefined') {
        return {};
    }

    var targetMetadata = {};
    for(var i = 0; i < restriction.length; i++) {
        if(restriction[i]['type'] === type) {
            targetMetadata = restriction[i];
        }
    }

    return targetMetadata;
}
//module.exports = findRestriction;

/**
 * 値制約より下限値を抽出
 * @param {Object} restriction 値制約
 * @return {string} 下限値
 */
function getMinOccur(restriction) {
    var cardinarity = findRestriction(restriction, 'cardinality');

    if (typeof(cardinarity['min']) === 'undefined') {
        return 0;
    }

    return cardinarity['min'];
}
//module.exports = getMinOccur;

/**
 * 値制約より上限値を抽出
 * @param {Object} restriction 値制約
 * @return {string} 上限値
 */
function getMaxOccur(restriction) {
    var cardinarity = findRestriction(restriction, 'cardinality');

    if (typeof(cardinarity['max']) === 'undefined') {
        return 'unbounded';
    }

    return cardinarity['max'];
}
//module.exports = getMaxOccur;

/**
 * 語彙のメタデータからCreatorのグループを抽出
 * @param {Array<Object>} metadata 語彙のメタデータArray
 * @return {Array<Object>} 分類されたCreatorのArray
 */
function getCreatorGroups(metadata) {
    var creatorGroups = [];
    var temp = {};
    for (var i = 0; i < metadata.length; i++) {
        if(metadata[i]['type'].match(/^creator.*$/)) {
            if (typeof(temp[metadata[i]['group']]) === 'undefined') temp[metadata[i]['group']] = [];
            temp[metadata[i]['group']].push(metadata[i]);
        }
    }
    var tempKeys = Object.keys(temp);
    for (var i = 0; i < tempKeys.length; i++) {
        creatorGroups.push(temp[tempKeys[i]]);
    }
    return creatorGroups;
}
//module.exports = getCreatorGroups;

/**
 * 語彙のクラスの配列から識別子検索する
 * @param {Array<Object>} classes 検索対象クラス配列
 * @param {String} identifier クラス識別子
 * @return {Object} クラス
 */
function findClass(classes, identifier) {
    for(var i = 0; i < classes.length; i++) {
        if (classes[i]['prefix'] + ':' + classes[i]['name'] === identifier) {
            return classes[i]
        }
    }
    return null;
}
//module.exports = findClass;