if (global) {
    var chai = require('chai');
    var expect = require('chai').expect;
    var chaiXml = require('chai-xml');
    chai.use(chaiXml);
    var IMILib2017Common = require('../lib/imi-lib-2017-common');
    var IMILib2017Ext = require('../index');
    var fs = require("fs");
    var dir = __dirname + "/test-vocabulary/";
    var xsd = new IMILib2017Common.Vocabulary(fs.readFileSync(dir + "xsd.imiv", "UTF-8").replace(/^\uFEFF/, ""));
    var imicoreXsd = fs.readFileSync(dir + "imicore242.xsd", "UTF-8");
}
  
  
describe('IMILib2017Common.Vocabulary', function() {
    describe('語彙の読み込み機能', function() {
        it('コア語彙2.4.1が読み込めること', function() {
            //var v = new IMILib2017Common.Vocabulary(fs.readFileSync(dir + "xsd.imiv.txt", "UTF-8").replace(/^\uFEFF/, ""), xsd);
            expect(function() {
                var v = new IMILib2017Common.Vocabulary(fs.readFileSync(dir + "imicore241.imiv", "UTF-8").replace(/^\uFEFF/, ""), xsd);
            }).not.to.throw();
            //expect(v.validate(IMIToolsCore.Vocabulary.LEVEL_WARN)).to.be.empty;
        });
        it('コア語彙2.4.2が読み込めること', function() {
            //var v = new IMILib2017Common.Vocabulary(fs.readFileSync(dir + "xsd.imiv.txt", "UTF-8").replace(/^\uFEFF/, ""), xsd);
            expect(function() {
                var v = new IMILib2017Common.Vocabulary(fs.readFileSync(dir + "imicore242.imiv", "UTF-8").replace(/^\uFEFF/, ""), xsd);
            }).not.to.throw();
            //expect(v.validate(IMIToolsCore.Vocabulary.LEVEL_WARN)).to.be.empty;
        });
    });
});

describe('IMILib2017Ext.Xsd', function() {
    describe('XSD出力機能', function() {
        it('コア語彙2.4.2のXMLスキーマが出力できること', function() {
            expect(function () {
                var xsd = new IMILib2017Ext.Xsd(fs.readFileSync(dir + "xsd.imiv", "UTF-8").replace(/^\uFEFF/, ""), false);
                var core = new IMILib2017Ext.Xsd(fs.readFileSync(dir + "imicore242.imiv", "UTF-8").replace(/^\uFEFF/, ""), false, xsd);
                var test =  core.toXsd();
            }).not.to.throw();
        })
        it('コア語彙2.4.2のXMLスキーマがXMLとして正しいこと', function() {
            var xsd = new IMILib2017Ext.Xsd(fs.readFileSync(dir + "xsd.imiv", "UTF-8").replace(/^\uFEFF/, ""), false);
            var core = new IMILib2017Ext.Xsd(fs.readFileSync(dir + "imicore242.imiv", "UTF-8").replace(/^\uFEFF/, ""), false, xsd);
            var output = core.toXsd();
            expect(output).xml.to.be.valid();
        });
        it('コア語彙2.4.2のXMLスキーマが正しく出力できる', function() {
            var xsd = new IMILib2017Ext.Xsd(fs.readFileSync(dir + "xsd.imiv", "UTF-8").replace(/^\uFEFF/, ""), false);
            var core = new IMILib2017Ext.Xsd(fs.readFileSync(dir + "imicore242.imiv", "UTF-8").replace(/^\uFEFF/, ""), false, xsd);
            var output = core.toXsd();
            expect(output).xml.to.equal(imicoreXsd);
        });
    });
});